/* ***** BEGIN LICENSE BLOCK *****
 * Version: MPL 1.1/GPL 2.0/LGPL 2.1
 *
 * The contents of this file are subject to the Mozilla Public License Version
 * 1.1 (the "License"); you may not use this file except in compliance with
 * the License. You may obtain a copy of the License at
 * http://www.mozilla.org/MPL/
 *
 * Software distributed under the License is distributed on an "AS IS" basis,
 * WITHOUT WARRANTY OF ANY KIND, either express or implied. See the License
 * for the specific language governing rights and limitations under the
 * License.
 *
 * The Original Code is Paste and Go.
 *
 * The Initial Developer of the Original Code is Jens Bannmann.
 * Portions created by the Initial Developer are Copyright (C) 2003
 * the Initial Developer. All Rights Reserved.
 *
 * Contributor(s) (alphabetical order):
 *  Jens Bannmann <jens.b@web.de>
 *
 * French version by :
 *  Guillaume Debeneix <guillaume.debeneix@etu.u-cergy.fr>
 *
 * Alternatively, the contents of this file may be used under the terms of
 * either the GNU General Public License Version 2 or later (the "GPL"), or
 * the GNU Lesser General Public License Version 2.1 or later (the "LGPL"),
 * in which case the provisions of the GPL or the LGPL are applicable instead
 * of those above. If you wish to allow use of your version of this file only
 * under the terms of either the GPL or the LGPL, and not to allow others to
 * use your version of this file under the terms of the MPL, indicate your
 * decision by deleting the provisions above and replace them with the notice
 * and other provisions required by the GPL or the LGPL. If you do not delete
 * the provisions above, a recipient may use your version of this file under
 * the terms of any one of the MPL, the GPL or the LGPL.
 *
 * ***** END LICENSE BLOCK ***** */

var pgInitDone = false;
window.addEventListener("load", pgStartup, true);

const pgClipboard = Components.classes["@mozilla.org/widget/clipboard;1"]
                    .createInstance(Components.interfaces.nsIClipboard);
const pgTrans = Components.classes["@mozilla.org/widget/transferable;1"]
                .createInstance(Components.interfaces.nsITransferable);
pgTrans.addDataFlavor("text/unicode");

var pgUrlbar;
var pgSearchbar;
var pgPlaceAtBottom = false;

function pgStartup(e) {
  if (pgInitDone) return;
  pgInitDone = true;

  const pgPref = Components.classes["@mozilla.org/preferences-service;1"]
                 .getService(Components.interfaces.nsIPrefService)
                 .getBranch("extensions.pasteandgo.");
  var pgGoKey = "V";
  var pgSearchKey = "S";
  try {
    pgPlaceAtBottom = pgPref.getBoolPref("placeAtBottom");
    pgGoKey = pgPref.getCharPref("GoHotkey");
    pgSearchKey = pgPref.getCharPref("SearchHotkey");
  } catch(e) {
    pgPref.setBoolPref("placeAtBottom", pgPlaceAtBottom)
    pgPref.setCharPref("GoHotkey", pgGoKey);
    pgPref.setCharPref("SearchHotkey", pgSearchKey);
  }

  pgUrlbar = document.getElementById("urlbar");
  dockElement("keyset", "key", "PasteAndGoKey",
      new Array("modifiers",   "key",   "command"  ),
      new Array("accel shift", pgGoKey, "PasteAndGoCommand"));
  dockElement("commandset", "command", "PasteAndGoCommand",
      new Array("oncommand"),
      new Array("PasteAndGo(false);"));
  pgUrlbar.addEventListener("popupshowing", pgInstallGo, true);

  pgSearchbar = document.getElementById("search-bar");
  if (pgSearchbar) {
    dockElement("keyset", "key", "PasteAndSearchKey",
        new Array("modifiers",   "key",       "command"  ),
        new Array("accel shift", pgSearchKey, "PasteAndSearchCommand"));
    dockElement("commandset", "command", "PasteAndSearchCommand",
        new Array("oncommand"),
        new Array("PasteAndGo(true);"));
    pgSearchbar.addEventListener("popupshowing", pgInstallSearch, true);
  }
}

//Version franaise par Guillaume DEBENEIX <guillaume.debeneix@etu.u-cergy.fr>
function pgInstallGo(e) {
  pgInstallMenuItem(e, "PasteAndGoMenuitem", "Coller et aller",
                    "PasteAndGoCommand");
}

function pgInstallSearch(e) {
  pgInstallMenuItem(e, "PasteAndSearchMenuitem", "Coller et rechercher", 
                    "PasteAndSearchCommand");
}
//Fin Version franaise

function pgInstallMenuItem(e, aID, aLabel, aCommand) {
  if (e.originalTarget.localName == "menupopup"
      && !document.getElementById(aID)) {
    var mi = document.createElement("menuitem");
    mi.setAttribute("id", aID);
    mi.setAttribute("label", aLabel);
    mi.setAttribute("command", aCommand);
    if (pgPlaceAtBottom) {
      e.originalTarget.appendChild(document.createElement("menuseparator"));
      e.originalTarget.appendChild(mi);
    } else {
      var items = e.originalTarget.childNodes;
      for (var i=0; i < items.length; i++) {
        var cmd = items[i].getAttribute("cmd");
        if(cmd == "cmd_paste") {
          e.originalTarget.insertBefore(mi, items[i+1]);
          break;
        }
      }
    }
  }
}

function dockElement(setType, type, id, aNames, aValues) {
  var sets = document.getElementsByTagName(setType);
  var set;
  if (sets && sets.length > 0) {
    set = sets[0];
  } else {
    set = document.createElement(setType);
    document.documentElement.appendChild(set);
  }
  var element = document.createElement(type);
  element.setAttribute("id", id);
  for (i=0; i < aNames.length; i++) {
    element.setAttribute(aNames[i], aValues[i]);
  }
  set.appendChild(element);
}

function PasteAndGo(doSearch) {
  pgClipboard.getData(pgTrans, pgClipboard.kGlobalClipboard);

  var str=new Object();
  var strLength=new Object();

  try {
    pgTrans.getTransferData("text/unicode", str, strLength);
  } catch (e) {
    alert("Clipboard is empty, nothing to paste!");
  }

  if (!str) { return; }
  if ("nsISupportsWString" in Components.interfaces) {
    str=str.value.QueryInterface(Components.interfaces.nsISupportsWString);
  } else if ("nsISupportsString" in Components.interfaces) {
    str=str.value.QueryInterface(Components.interfaces.nsISupportsString);
  } else {
    return;
  }
  if (!str) { return; }
  if (doSearch) {
    // put the query in the search bar
    pgSearchbar.setAttribute("value", str);

    if (pgSearchbar.searchValue) {
      // in old builds (up to 0.7), searchValue's getter is so kind and gives
      // us the URL to load:
      str = pgSearchbar.searchValue;
      pgUrlbar.setAttribute("value", str);
      loadURI(str);
    } else {
      // in current builds (0.8+), pgSearchbar no longer returns "searchValue",
      // but has a method named "onTextEntered" which starts the search
      pgSearchbar.onTextEntered();
    }

  } else {
    pgUrlbar.setAttribute("value", str);
    loadURI(str);
  }
}
